/*******************************************************************
   Ersteller:         Roger Strauss
   Letzte Aenderung:  07.07.2021
   Revision:          V1.0

  https://elektronik-strauss.de/

  Programm zum steuern 2er WS2812B Stripes mit einem Nano BLE von Keywish
 ********************************************************************/

//WS2812B Initialisierung
#include <WS2812FX.h>

//******** Einstellungen durch Nutzer ***********//

#define LED_COUNT 37        //Anzahl der LEDs
#define LED_PIN1 2          //Pins mit denen die Stripes verbunden sind
#define LED_PIN2 3

//******** Einstellungen durch Nutzer ***********//

#define rot 70
#define gruen 71
#define blau 72
#define weiss 73
#define schwarz 74
#define gelb 75
#define cyan 76
#define magenta 77
#define lila 78
#define orange 79
#define pink 80
#define grau 81
#define ultraweiss 82


WS2812FX streifen1 = WS2812FX(LED_COUNT, LED_PIN1, NEO_GRB + NEO_KHZ800);
WS2812FX streifen2 = WS2812FX(LED_COUNT, LED_PIN2, NEO_GRB + NEO_KHZ800);


//WS2812B Initialisierung

//Initialisierung EEPROM
#include <EEPROM.h>
#define EepromAdresseHelligkeit 0
#define EepromAdresseGeschwindigkeit 1
#define EepromAdresseModus 2
#define EepromAdresseFarbe 3
#define EepromAdresseVerzoegerung 4

byte WS2812_helligkeit, WS2812_modus, WS2812_farbe, WS2812_geschwindigkeit, WS2812_verzoegerung;
//Initialisierung EEPROM

byte WS2812_aktualisieren;
unsigned long letzteZeitWS2812_aktualisieren;


void setup()
{
  Serial.begin(9600);

  // Holen der letzten Speicherstände
  WS2812_helligkeit = EEPROM.read(EepromAdresseHelligkeit);
  WS2812_geschwindigkeit = EEPROM.read(EepromAdresseGeschwindigkeit);
  WS2812_farbe = EEPROM.read(EepromAdresseFarbe);
  WS2812_modus = EEPROM.read(EepromAdresseModus);
  WS2812_verzoegerung = EEPROM.read(EepromAdresseVerzoegerung);
  
  WS2812_aktualisieren = 1;
  streifen1.init();
  streifen2.init();

  Serial.println("AufGehts...");
}


void loop()
{
  // Ständige Aktualisieren 
  streifen1.service();
  streifen2.service();
  RefreshApp();

// Wird durchlaufen bei einer Änderung der Einstellungen
  if (WS2812_aktualisieren == 1)
  { //LED-Streifen1 starten //LED-Streifen2 stoppen
    streifen1.stop();
    streifen2.stop();

    streifen1.setBrightness(map(WS2812_helligkeit, 200, 249, 30, 100));
    streifen1.setSpeed(map(WS2812_geschwindigkeit, 150, 199, 50, 2000));

    switch (WS2812_farbe)
    {
      case rot: streifen1.setColor(RED); break;
      case gruen: streifen1.setColor(GREEN); break;
      case blau: streifen1.setColor(BLUE); break;
      case weiss: streifen1.setColor(WHITE); break;
      case schwarz: streifen1.setColor(BLACK); break;
      case gelb: streifen1.setColor(YELLOW); break;
      case cyan: streifen1.setColor(CYAN); break;
      case magenta: streifen1.setColor(MAGENTA); break;
      case lila: streifen1.setColor(PURPLE); break;
      case orange: streifen1.setColor(ORANGE); break;
      case pink: streifen1.setColor(PINK); break;
      case grau: streifen1.setColor(GRAY); break;
      case ultraweiss: streifen1.setColor(ULTRAWHITE); break;
    }

    streifen1.start();

    streifen1.setMode(WS2812_modus);

    letzteZeitWS2812_aktualisieren = millis();
    WS2812_aktualisieren = 2;
    Serial.println("aktualisieren 1");

  }

  //Wird durchlaufen bei einer Änderung und Ablauf der Eingestellten Verzörgerung
  if (WS2812_aktualisieren == 2 && millis() > letzteZeitWS2812_aktualisieren + map(WS2812_verzoegerung, 100, 149, 0, 2000))
  { //LED-Streifen2 starten
    streifen2.setBrightness(map(WS2812_helligkeit, 200, 249, 30, 100));
    streifen2.setSpeed(map(WS2812_geschwindigkeit, 150, 199, 50, 2000));

   switch (WS2812_farbe)
    {
      case rot: streifen2.setColor(RED); break;
      case gruen: streifen2.setColor(GREEN); break;
      case blau: streifen2.setColor(BLUE); break;
      case weiss: streifen2.setColor(WHITE); break;
      case schwarz: streifen2.setColor(BLACK); break;
      case gelb: streifen2.setColor(YELLOW); break;
      case cyan: streifen2.setColor(CYAN); break;
      case magenta: streifen2.setColor(MAGENTA); break;
      case lila: streifen2.setColor(PURPLE); break;
      case orange: streifen2.setColor(ORANGE); break;
      case pink: streifen2.setColor(PINK); break;
      case grau: streifen2.setColor(GRAY); break;
      case ultraweiss: streifen2.setColor(ULTRAWHITE); break;
    }


    streifen2.start();
    streifen2.setMode(WS2812_modus);

    WS2812_aktualisieren = 0;
    Serial.println("aktualisieren 2");
  }
}

void SaveLedEinstellungen()
{
  EEPROM.write(EepromAdresseHelligkeit, WS2812_helligkeit);
  EEPROM.write(EepromAdresseGeschwindigkeit, WS2812_geschwindigkeit);
  EEPROM.write(EepromAdresseFarbe, WS2812_farbe);
  EEPROM.write(EepromAdresseModus, WS2812_modus);
  EEPROM.write(EepromAdresseVerzoegerung, WS2812_verzoegerung);
}

void RefreshApp()
{
  byte in_byte;

  // bei einem Arduino Uno
  //if (Serial.available() > 0 )

  // bei einem Arduino Mega
  //if (Serial1.available() > 0 )

  //ESP 32
  //if (SerialBT.available() > 0)

  //Nano-BLE
  while (Serial.available() > 0)
  {
    // bei einem Arduino Uno
    //in_byte = Serial.read();

    // bei einem Arduino Mega
    //in_byte = Serial1.read();

    // bei einem ESP32
    //in_byte = SerialBT.read();
    //
    //Nano-BLE
    in_byte = Serial.read();

    Serial.println(in_byte);
    if (in_byte != 0)
    {
      if (in_byte < 70)
        WS2812_modus = in_byte - 1;
      else if (in_byte < 100)
        WS2812_farbe = in_byte;
      else if (in_byte < 150)
        WS2812_verzoegerung = in_byte;
      else if (in_byte < 200)
        WS2812_geschwindigkeit = in_byte;
      else if (in_byte < 250)
        WS2812_helligkeit = in_byte;
      else if (in_byte == 255)
        SaveLedEinstellungen();
      else
        Serial.println("Fehlerhaftes Signal App");

      WS2812_aktualisieren = 1;
    }
  }
}
